from pathlib import Path

# TODO - refactor into object

# input - base path, exclude directories, exclude files
# output - list of all files that are not excluded or in excluded directories


def in_stem(fs_element, excluded_dirs):
    for parent in fs_element.parents:
        if parent in excluded_dirs:
            return True
    return False


def get_files(base_path, excluded_dirs=None, excluded_files=None):
    files = []
    for file in base_path.rglob('*'):
        file_rel_path = file.relative_to(base_path)
        if not in_stem(file_rel_path, excluded_dirs) \
                and file_rel_path not in excluded_files \
                and file.is_file():
            files.append(file_rel_path)
    return files

if __name__ == "__main__":

    dir_path = '/Users/kobarhan/workspace/igit_test'
    repo_path = Path(dir_path)

    ex_dirs = [Path(ex_dir) for ex_dir in ['.git']]
    ex_files = [Path(ex_file) for ex_file in ['data3.txt']]

    files_objects = get_files(repo_path, ex_dirs, ex_files)
    print(files_objects)

    files_objects = [str(file) for file in files_objects]
    print(files_objects)



import os
import emoji
from emoji import EMOJI_ALIAS_UNICODE
from termcolor import colored


class Display:

    def __init__(self, os_type=os.name) -> None:
        self.os_type = os_type

    def message(self, text, color='white', icon='poop'):
        _emoji, _emoji_render_method = self.emojize(icon)
        self._display(f'{_emoji} - {text}', color, _emoji_render_method)

    def list(self, text, items, color, icon):
        _emoji, _emoji_render_method = self.emojize(icon)
        self._display(f'{_emoji} - {text}:', color, _emoji_render_method)
        for item in items:
            self._display(" " * 8 + item, 'white', _emoji_render_method)

    def emojize(self, icon):
        icon = f':{icon}:'
        if self.os_type == 'nt':
            import psutil
            res_icon = '*'
            shell = psutil.Process(os.getpid()).parent().parent().name()
            if shell == 'powershell.exe':
                res_icon = EMOJI_ALIAS_UNICODE[icon]
            return res_icon, lambda *args, **kwargs: args
        else:
            return icon, emoji.emojize

    @classmethod
    def diff(cls, diff):
        if diff.startswith('+++') or diff.startswith('---'):
            print(colored(diff, 'white'))
        elif diff.startswith('+'):
            print(colored(diff, 'green'))
        elif diff.startswith('-'):
            print(colored(diff, 'red'))
        else:
            print(colored(diff, 'white'))

    @staticmethod
    def _display(text, color, emoji_method=lambda *args, **kwargs: args):
        print(colored(emoji_method(text, use_aliases=True), color))


if __name__ == '__main__':

    display = Display(os_type='nt')
    display.message('hello', 'green', 'thumbsup')
    display.list('hello', ['item1', 'item2'], 'green', 'thumbsup')



import os
from invoke import run


class Shell:

    def __init__(self, silent=False, directory=None) -> None:
        self.silent = silent
        self.directory = directory

    def __rshift__(self, cmd):
        if self.directory:
            os.chdir(self.directory)
        result = run(cmd, hide=True, warn=True)
        if not result.ok:
            raise Exception(f'failed to run command: {cmd}\n'
                            f'{result.stderr}')
        if not self.silent:
            print(result.stdout)
        return result


if __name__ == "__main__":
    shell = Shell()
    shell >> "ls -la"



import os

import requests
from git import Repo
from git import InvalidGitRepositoryError
from igit.interactive.display import Display
from igit.interactive.interact import Interact


class GitOps:

    change_types = {
        'unstaged': None,
        'staged': 'HEAD'
    }

    def __init__(self, path=os.getcwd()) -> None:
        try:
            self.repo = Repo(path, search_parent_directories=True)
            self.repo_path = os.path.dirname(self.repo.git_dir)
            self.branch = self.repo.active_branch
            self.display = Display()
            self.interact = Interact()

        except InvalidGitRepositoryError:
            raise InvalidGitRepositoryError('Not a git repo, I have no power here...')

    def get_changed_files(self, change):
        return [item.a_path for item in self.repo.index.diff(self.change_types[change])]

    def get_untracked_files(self):
        return self.repo.untracked_files

    def get_all_changes(self):
        return self.get_changed_files('staged') + \
               self.get_changed_files('unstaged') + \
               self.get_untracked_files()

    def calculate_change_list(self, diff_types=None):
        change_list = []
        for diff_type in diff_types:
            change_list += [item.a_path for item
                            in self.repo.index.diff(self.change_types[diff_type])]
        change_list += self.repo.untracked_files
        return change_list

    def switch_branch(self, branch_name):
        self.repo.git.checkout(branch_name)
        self.branch = self.repo.active_branch.name
        self.display.message(f'Switched to branch: {self.branch}', 'yellow', 'checkered_flag')

    def create_branch(self, branch_name):
        self.repo.create_head(branch_name)
        self.switch_branch(branch_name)

    def create_gitignore(self, gitignore_path):
        if os.path.exists(gitignore_path):
            self.display.message('Attention - gitignore file detected,'
                                 'creating a new one will overwrite existing',
                                 color='red',
                                 icon='exclamation')
        rv = self.interact.confirm('Create from template [y] or blank [N]')
        if rv is None:
            return
        if rv:
            gitignore_files = ['Python', 'Java', 'Go', 'Swift', 'Node', 'C', 'C++']
            rv = self.interact.select('What kind of .gitignore do you need', gitignore_files)
            url = f'https://raw.githubusercontent.com/github/gitignore/master/{rv}.gitignore'
            r = requests.get(url)
            with open(gitignore_path, 'wb') as f:
                f.write(r.content)
        else:
            open(gitignore_path, 'a').close()
        self.display.message('Created .gitignore', icon='tada')
        return


def in_gitignore(gitignore_path, item):
    gitignore = []
    with open(gitignore_path, 'r') as ignore_file:
        for line in ignore_file.readlines():
            gitignore.append(line.strip())
    return item in gitignore


def add_gitignore(gitignore_path, item):
    with open(gitignore_path, 'a') as ignore_file:
        ignore_file.write('\n')
        ignore_file.write(item)


if __name__ == '__main__':
    dir_path = '/Users/kobarhan/workspace/igit_test'
    gc = GitOps(dir_path)

    staged = gc.get_changed_files('staged')
    unstaged = gc.get_changed_files('unstaged')
    untracked = gc.get_untracked_files()

    print(f'staged changes: {staged}')
    print(f'unstaged changes: {unstaged}')
    print(f'untracked changes: {untracked}')

    print(f'all changes: {gc.get_all_changes()}')



import os
from pathlib import Path

import click
from click_help_colors import HelpColorsGroup
from git import InvalidGitRepositoryError

from igit.core.commands import Igit
from igit.interactive.display import Display


@click.group(
    cls=HelpColorsGroup,
    help_headers_color='yellow',
    help_options_color='magenta',
    help_options_custom_colors={
        'up': 'cyan',
        'push': 'cyan',
        'undo': 'red',
        'unstage': 'red',
        'revert': 'red',
        'diff': 'green',
        'branch': 'green',
        'add': 'blue',
        'commit': 'blue',
        'save': 'blue',
    }
)
def cli():
    """\b
_____        __________
___(_)______ ___(_)_  /_
__  /__  __ `/_  /_  __/
_  / _  /_/ /_  / / /_
/_/  _\__, / /_/  \__/
     /____/
    """


@cli.command(
    help='Add unstaged files.'
)
@click.option('--file', default=[], help='file to add', multiple=True)
@click.option('--all', '-a', is_flag=True, default=False, help='add all unstaged files')
def add(file, all):
    Igit().add(file, all)


@cli.command(help='Commit changes.')
@click.option('--message', '-m', default=None, help='commit message')
@click.option('--add', '-a', is_flag=True, default=False, help='add before commit')
def commit(message, add):
    Igit().commit(message, add)


@cli.command(help='Push changes.')
@click.option('--add', '-a', is_flag=True, default=False, help='add --all and commit before push')
@click.option('--commit', '-c', is_flag=True, default=False, help='push to remote')
def push(add, commit):
    Igit().push(add, commit)


@cli.command(help='Adds and Commits changes.')
@click.option('--message', default=None, help='commit message (optional)')
def save(message):
    Igit().save(message)


@cli.command(help='Switch to another branch.')
@click.option('--name', '-n', default=None, help='target branch to switch to')
@click.option('--hopping_on', '-h', is_flag=True, default=False, help='activate branch hopping')
@click.option('--create', '-c', is_flag=True, default=False, help='create a new branch')
def branch(name, hopping_on, create):
    return Igit().branch(name, hopping_on, create)


@cli.command(help='Adds, Commits and Pushes changes to remote.')
@click.option('--message', default=None, help='commit message (optional)')
def up(message):
    Igit().up(message)


@cli.command(help='Prints diff of selected file.')
def diff():
    return Igit().diff()


@cli.command(help='Undo un-staged (non added) changes.')
@click.option('--file', default=[], help='file to add', multiple=True)
@click.option('--all', is_flag=True, default=False, help='undo all unstaged changes')
def undo(file, all):
    return Igit().undo(file, all)


@cli.command(help='Unstage changes.')
@click.option('--file', default=[], help='file to add', multiple=True)
@click.option('--all', is_flag=True, default=False, help='unstage all files')
def unstage(file, all):
    return Igit().unstage(file, all)


@cli.command(help='Revert commit (NOT IMPLEMENTED).')
def revert():
    # TODO - implement
    return 'NOT IMPLEMENTED'


@cli.command(help='Rename current branch.')
@click.argument('name')
def rename(name):
    return Igit().rename(name)


@cli.command(help='Perform .gitignore operations.')
@click.option('--reset', is_flag=True, default=False, help='resets gitignore')
@click.option('--create', is_flag=True, default=False, help='create gitignore filr from template')
def ignore(reset, create):
    return Igit().ignore(reset, create)


@cli.command(help='Prints igit version.')
def version():
    here = Path(__file__).parent.absolute()
    package_conf = {}
    with open(os.path.join(here, "__version__.py")) as f:
        exec(f.read(), package_conf)
    print(package_conf['__version__'])


def run():
    try:
        cli(prog_name='igit')
    except InvalidGitRepositoryError as e:
        Display().message(e, 'yellow', 'sweat_smile')
    except Exception as e:
        Display().message(f'Encountered an error: {e}', 'red', 'x')


if __name__ == "__main__":
    run()



import random
import string


from pylint.lint import Run


file_name_prefix = 'file_'


def random_str(length):
    letters = string.ascii_lowercase
    return ''.join(random.choice(letters) for _ in range(length))


def run_linter(module, pylintrc_path, threshold):
    lint_run = Run([module, f'--rcfile={pylintrc_path}'], exit=False)
    score = lint_run.linter.stats['global_note']
    assert score > threshold, f'FAILED: linting score for module {module} too low: {score}'



import os
import pytest
from click.testing import CliRunner


from igit.core.commands import Igit
from igit.core.shell_ops import Shell
from igit.cli import add, unstage, undo, commit


@pytest.fixture()
def source_dir():
    return os.environ['source_dir']


@pytest.fixture()
def test_dir():
    return os.environ['test_dir']


@pytest.fixture()
def igit(test_dir):
    return Igit(test_dir)


def test_add(igit):
    shell = Shell(directory=igit.gitops.repo_path)
    shell >> "./make_changes.sh 2"
    unstaged_files = len(igit.gitops.get_changed_files('unstaged'))
    assert unstaged_files == 1

    runner = CliRunner()
    result = runner.invoke(add, ['--all'])
    assert result.exit_code == 0

    unstaged_files = len(igit.gitops.get_changed_files('unstaged'))
    assert unstaged_files == 0
    staged_files = len(igit.gitops.get_changed_files('staged'))
    assert staged_files == 1

    # clean up - unstage
    result = runner.invoke(unstage, ['--all'])
    assert result.exit_code == 0
    staged_files = len(igit.gitops.get_changed_files('staged'))
    assert staged_files == 0

    # clean up - undo
    result = runner.invoke(undo, ['--all'])
    assert result.exit_code == 0
    unstaged_files = len(igit.gitops.get_changed_files('unstaged'))
    assert unstaged_files == 0


def test_commit(igit):
    shell = Shell(directory=igit.gitops.repo_path)
    shell >> "./make_changes.sh 2"
    unstaged_files = len(igit.gitops.get_changed_files('unstaged'))
    assert unstaged_files == 1

    runner = CliRunner()
    result = runner.invoke(add, ['--all'])
    assert result.exit_code == 0

    unstaged_files = len(igit.gitops.get_changed_files('unstaged'))
    assert unstaged_files == 0
    staged_files = len(igit.gitops.get_changed_files('staged'))
    assert staged_files == 1

    result = runner.invoke(commit, ['-m', 'test commit'])
    assert result.exit_code == 0

    staged_files = len(igit.gitops.get_changed_files('staged'))
    assert staged_files == 0
    unstaged_files = len(igit.gitops.get_changed_files('unstaged'))
    assert unstaged_files == 0



import os


os_type = os.name
if os_type == 'nt':
    from PyInquirer import prompt
else:
    from inquirer import prompt, List, Checkbox, Confirm, Text


class Interact:

    def __init__(self, os_name=os.name) -> None:
        self.os_type = os_name

    def confirm(self, message):
        inquiry = Inquiry('confirm', message, 'confirm', os_name=os_type).inquiry
        return self.response(prompt(inquiry), 'confirm')

    def text(self, message):
        inquiry = Inquiry('text', message, 'text', os_name=os_type).inquiry
        return self.response(prompt(inquiry), 'text')

    def select(self, message, choices):
        inquiry = Inquiry('select', message, 'list', choices=choices, os_name=os_type).inquiry
        return self.response(prompt(inquiry), 'select')

    def choose(self, message, choices):
        inquiry = Inquiry('choose', message, 'checkbox', choices=choices, os_name=os_type).inquiry
        return self.response(prompt(inquiry), 'choose')

    @staticmethod
    def response(res, opt):
        return res[opt] if res not in [None, {}] else None


class Inquiry:

    def __init__(self, name, message, inq_type, choices=None, os_name=os.name) -> None:

        inquiry = {
            'name': name,
            'message': message,
            'choices': choices
        }

        if os_name == 'nt':
            inquiry.update({'type': inq_type})
            if inq_type == 'confirm':
                inquiry.update({'default': False})
            elif inq_type == 'checkbox':
                inquiry['choices'] = [{'name': choice} for choice in choices]
            self.inquiry = [inquiry]
        else:
            inquiry_types = {
                'list': List,
                'checkbox': Checkbox,
                'confirm': Confirm,
                'text': Text
            }
            self.inquiry = [inquiry_types[inq_type](**inquiry)]


if __name__ == '__main__':
    print(Interact('nt').confirm('yes or no'))
    print(Interact().text('enter some input'))
    print(Interact('nt').select('please choose', ['option 1', 'option 2']))
    print(Interact('nt').choose('please choose', ['option 1', 'option 2']))



import os
from pathlib import Path

import requests
from git import InvalidGitRepositoryError
from git import GitCommandError

from igit.core.fs_ops import get_files
from igit.interactive.display import Display
from igit.interactive.interact import Interact
from igit.core.shell_ops import Shell
from igit.core.config import *

from igit.core.git_ops import in_gitignore, add_gitignore, GitOps


class Igit:

    def __init__(self, path=os.getcwd()):
        try:
            self.gitops = GitOps(path)
            self.display = Display()
            self.interact = Interact()
        except InvalidGitRepositoryError:
            raise

    def add(self, _files, _all):
        try:
            unstaged = self.gitops.get_changed_files('unstaged') + \
                       self.gitops.get_untracked_files()
            if unstaged:
                if _all:
                    selected = unstaged
                elif _files:
                    selected = self._intersection(_files, unstaged)
                else:
                    selected = self.interact.choose('choose files to add', unstaged)
                if selected:
                    self.gitops.repo.git.add(selected)
                    self.display.list('added', selected, 'green', 'thumbsup')
                else:
                    self.display.message('command had no effect', 'yellow', 'speak_no_evil')
            else:
                self.display.message('nothing to add', 'yellow', 'speak_no_evil')
        except GitCommandError as e:
            self.display.message(f'unable to add\n{e}', 'red', 'x')

    def commit(self, _message, _add):
        if not _message:
            prompt = self.interact.text(f'commit message [{DEFAULT_COMMIT}]')
            _message = prompt if prompt else DEFAULT_COMMIT
        if _add:
            self.add(_files=None, _all=True)
        try:
            if len(self.gitops.repo.index.diff("HEAD")) > 0:
                commit_resp = self.gitops.repo.git.commit('-m', _message)
                self.display.message(commit_resp, 'white', 'cat')
                self.display.message('committed', 'yellow', 'lock')
            else:
                self.display.message('nothing to commit', 'yellow', 'lock')
        except GitCommandError as e:
            self.display.message(f'unable to commit\n{e}', 'red', 'x')

    def push(self, _add, _commit):
        if _add:
            self.add(_files=None, _all=False)
            self.commit(_message=None, _add=False)
        elif _commit:
            self.commit(_message=None, _add=False)
        try:
            self.gitops.repo.git.push('origin', self.gitops.branch)
            self.display.message('pushed', 'cyan', 'rocket')
        except GitCommandError as e:
            self.display.message(f'unable to push\n{e}', 'red', 'x')

    def save(self, message):
        self.add(_files=None, _all=True)
        message = message if message else DEFAULT_COMMIT
        self.commit(_message=message, _add=False)

    def up(self, message):
        self.add(_files=None, _all=True)
        message = message if message else DEFAULT_COMMIT
        self.commit(_message=message, _add=False)
        self.push(_add=False, _commit=False)

    def branch(self, target_branch, hopping_mode, create_new):
        """
        Swith to target_branch if specified, else prompt branch menu.
        Implements auto-stashing to allow flex branch hopping.
        :param target_branch:
        :return:
        """
        if create_new:
            branch_name = Interact().text('Give it a name')
            self.gitops.create_branch(branch_name)
            return

        if not target_branch:
            branches = [branch.name for branch in list(self.gitops.repo.branches)]
            if len(branches) == 1:
                create_new = Interact().confirm('No local branches detected. create new?')
                if create_new:
                    branch_name = Interact().text('Give it a name')
                    self.gitops.create_branch(branch_name)
                    return
                return
            else:
                target_branch = self.interact.select('choose terget branch', branches)
                if not target_branch:
                    return

        if hopping_mode:
            # handle branch hopping in case of untracked files
            untracked = self.gitops.get_untracked_files()
            if untracked:
                self.display.list('Untracked files found', untracked, 'white', 'speak_no_evil')
                if not self.interact.confirm(f'Stage them to continue?'):
                    self.display.message('Stage files to branch with igit', 'yellow', 'sweat_smile')
                    return
                else:
                    self.display.message('Staging untracked files', 'yellow', 'floppy_disk')
                    self.gitops.repo.git.add(untracked)

            change_list = self.gitops.get_all_changes()

            if len(change_list) > 0:
                self.gitops.repo.git.stash(f'save', f'{GLOBAL_STASH_PREFIX}_{self.gitops.branch}')
                self.display.message(f'Saving diff (stash): {GLOBAL_STASH_PREFIX}_{self.gitops.branch}', 'green', 'thumbsup')

        self.gitops.switch_branch(target_branch)

        if hopping_mode:
            stash_list = self.gitops.repo.git.stash('list')
            stash_stub = f'{GLOBAL_STASH_PREFIX}_{self.gitops.branch}'
            if stash_list:
                stash_list = stash_list.split("\n")

            for stash in stash_list:
                stash_name = stash.split(' ')[-1]
                if stash_stub == stash_name:
                    stash_index = stash.split(' ')[0].replace(':', '')
                    self.gitops.repo.git.stash(f'pop', stash_index)
                    self.display.message(f'Loading diff (stash): {stash_name}', 'green', 'thumbsup')
                    break

    def diff(self):
        change_list = self.gitops.get_changed_files('unstaged')
        if not change_list:
            self.display.message('No diff found', 'yellow', 'speak_no_evil')

        file = self.interact.select('select file to view diff', change_list)
        if not file:
            return
        diffs = self.gitops.repo.git.diff(file).split('\n')
        for diff in diffs:
            self.display.diff(diff)

    def rename(self, name):
        try:
            self.gitops.repo.active_branch.rename(name)
        except Exception as e:
            print(e)

    def undo(self, _files, _all):
        try:
            unstaged = self.gitops.get_changed_files('unstaged')
            if unstaged:
                if _all:
                    selected = unstaged
                elif _files:
                    selected = self._intersection(_files, unstaged)
                else:
                    selected = self.interact.choose('choose files to undo changes', unstaged)
                if selected:
                    self.gitops.repo.git.checkout('--', selected)
                    self.display.list('changes undone', selected, 'green', 'thumbsup')
                else:
                    self.display.message('command had no effect', 'yellow', 'speak_no_evil')
            else:
                self.display.message('nothing to undo', 'yellow', 'speak_no_evil')
        except GitCommandError as e:
            self.display.message(f'unable to undo\n{e}', 'red', 'x')

    def unstage(self, _files, _all):
        try:
            unstaged = self.gitops.get_changed_files('staged')
            if unstaged:
                if _all:
                    selected = unstaged
                elif _files:
                    selected = self._intersection(_files, unstaged)
                else:
                    selected = self.interact.choose('choose files to unstage', unstaged)
                if selected:
                    self.gitops.repo.git.reset('--', selected)
                    self.display.list('files unstaged', selected, 'green', 'thumbsup')
                else:
                    self.display.message('command had no effect', 'yellow', 'speak_no_evil')
            else:
                self.display.message('nothing to unstage', 'yellow', 'speak_no_evil')
        except GitCommandError as e:
            self.display.message(f'unable to unstage\n{e}', 'red', 'x')

    def ignore(self, reset, create):
        # check for .gitignore file, prompt to create if none detected
        gitignore_path = os.path.join(self.gitops.repo_path, '.gitignore')

        if create:
            self.gitops.create_gitignore(gitignore_path)
            return

        if not os.path.exists(gitignore_path):
            if self.interact.confirm('no gitignore file detected, create new?'):
                self.gitops.create_gitignore(gitignore_path)
                return

        if reset:
            shell = Shell()
            shell >> 'git rm -r --cached .'
            self.commit('.gitignore fixed by igit', True)

        else:
            ex_dirs = [Path(ex_dir) for ex_dir in ['.git']]
            opt_files = get_files(Path(self.gitops.repo_path), ex_dirs, [])
            opt_files = [str(file) for file in opt_files]

            # subtract files that are already added
            if not opt_files:
                return 'No files found to add.'
            else:
                files = self.interact.choose('select files to gitignore', opt_files)
                if files:
                    try:
                        for file in files:
                            if not in_gitignore(gitignore_path, file):
                                add_gitignore(gitignore_path, file)
                    except Exception as e:
                        print(f'No gitignore file found: {e}')
                else:
                    self.display.message('gitignore unchanged (use space key to select!)', 'yellow', 'speak_no_evil')


    @staticmethod
    def _intersection(iterable1, iterable2):
        return list(set(iterable1).intersection(iterable2))



import setuptools
from pathlib import Path
import pkg_resources

package_conf = {}
with open(Path("igit") / "__version__.py") as f:
    exec(f.read(), package_conf)
with open("README.md", "r") as fh:
    package_conf['description'] = fh.read()

with Path('requirements.txt').open() as requirements_txt:
    install_requires = [
        str(requirement)
        for requirement
        in pkg_resources.parse_requirements(requirements_txt)
    ]

setuptools.setup(
    name="igit",
    version=package_conf['__version__'],
    author="kobi",
    author_email="",
    description="Interactive git and more",
    long_description=package_conf['description'],
    long_description_content_type="text/markdown",
    url="https://github.com/kobibarhanin/igit",
    packages=setuptools.find_packages(),
    entry_points={
        "console_scripts": [
            "igit=igit.cli:run",
        ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=install_requires,
    python_requires='>=3.7',
)



from igit.tests.test_main import source_dir
from igit.tests.utils import run_linter


def test_linting(source_dir):
    run_linter('igit.core', f'{source_dir}/tests/.pylintrc', 8)
    run_linter('igit.interactive', f'{source_dir}/tests/.pylintrc', 8)
